import ctypes
import numpy as np

def _find_lib(lib_name):
    import os
    import sys
    for par in sys.path:
        if os.path.isfile(os.path.join(par, lib_name)):
            return os.path.join(par, lib_name)
    for par in sys.path:
        if os.path.isfile(os.path.join(par, 'seam_curving', lib_name)):
            return os.path.join(par, 'seam_curving', lib_name)
    return 'libMaxflow.so'


def seam_curving(runner, np_data_mat):
    len_x, len_y, len_z = np_data_mat.shape
    tmp = np_data_mat.flatten().astype(np.int64)
    ret = np.zeros((len_x * len_y * 3, ), dtype=np.int32)
    dataptr = tmp.ctypes.data_as(ctypes.c_void_p)
    outprt = ret.ctypes.data_as(ctypes.c_void_p)
    runner(dataptr, len_x, len_y, len_z, outprt)
    return ret.reshape((len_x * len_y, 3))
 
def seam_curving_forward(runner, np_data_mat):
    _, len_x, len_y, len_z = np_data_mat.shape
    tmp = np_data_mat.flatten().astype(np.int64)
    ret = np.zeros((len_x * len_y * 3, ), dtype=np.int32)
    dataptr = tmp.ctypes.data_as(ctypes.c_void_p)
    outprt = ret.ctypes.data_as(ctypes.c_void_p)
    runner(dataptr, len_x, len_y, len_z, outprt)
    return ret.reshape((len_x * len_y, 3))

def seam_curving_dinic(np_data_mat):
    lib = ctypes.cdll.LoadLibrary(_find_lib("libMaxflow.so"))
    return seam_curving(lib.seam_curving_dinic, np_data_mat)

def seam_curving_aor(np_data_mat):
    lib = ctypes.cdll.LoadLibrary(_find_lib("libMaxflow.so"))
    return seam_curving(lib.seam_curving_aor, np_data_mat)

def seam_curving_forward_dinic(np_data_mat):
    lib = ctypes.cdll.LoadLibrary(_find_lib("libMaxflow.so"))
    return seam_curving_forward(lib.seam_curving_forward_dinic, np_data_mat)

def seam_curving_forward_aor(np_data_mat):
    lib = ctypes.cdll.LoadLibrary(_find_lib("libMaxflow.so"))
    return seam_curving_forward(lib.seam_curving_forward_aor, np_data_mat)

def print_test(np_data_mat):
    lib = ctypes.cdll.LoadLibrary(_find_lib("libMaxflow.so"))

    len_x, len_y, len_z = np_data_mat.shape
    tmp = np_data_mat.flatten().astype(np.int64)
    ret = np.zeros((len_x * len_y * 3, ), dtype=np.int32)
    dataptr = tmp.ctypes.data_as(ctypes.c_void_p)
    outprt = ret.ctypes.data_as(ctypes.c_void_p)
    
    lib.print_test(dataptr, len_x, len_y, len_z)


def fill_test(np_data_mat):
    lib = ctypes.cdll.LoadLibrary(_find_lib("libMaxflow.so"))

    len_x, len_y, len_z = np_data_mat.shape
    tmp = np_data_mat.flatten().astype(np.int64)
    ret = np.zeros((len_x * len_y * 3, ), dtype=np.int32)
    dataptr = tmp.ctypes.data_as(ctypes.c_void_p)
    outprt = ret.ctypes.data_as(ctypes.c_void_p)
    
    lib.fill_test(outprt, len_x, len_y)
    return ret.reshape((len_x * len_y, 3))

